# Copyright (C) 2019 QuantStack and the Mamba contributors.
# Copyright (C) 2022 Anaconda, Inc
# Copyright (C) 2023 conda
# SPDX-License-Identifier: BSD-3-Clause

# TODO: Temporarily vendored from mamba.utils v0.19 on 2021.12.02
# Decide what to do with it when we split into a plugin
# 2022.02.15: updated vendored parts to v0.21.2
# 2022.11.14: only keeping channel prioritization and context initialization logic now

import logging
from functools import lru_cache
from importlib.metadata import version
from typing import Dict

import libmambapy as api
from conda.base.constants import ChannelPriority
from conda.base.context import context

log = logging.getLogger(f"conda.{__name__}")


@lru_cache(maxsize=1)
def mamba_version():
    return version("libmambapy")


def _get_base_url(url, name=None):
    tmp = url.rsplit("/", 1)[0]
    if name:
        if tmp.endswith(name):
            return tmp.rsplit("/", 1)[0]
    return tmp


def set_channel_priorities(index: Dict[str, "_ChannelRepoInfo"], has_priority: bool = None):
    """
    This function was part of mamba.utils.load_channels originally.
    We just split it to reuse it a bit better.
    """
    if not index:
        return index

    if has_priority is None:
        has_priority = context.channel_priority in [
            ChannelPriority.STRICT,
            ChannelPriority.FLEXIBLE,
        ]

    subprio_index = len(index)
    if has_priority:
        # max channel priority value is the number of unique channels
        channel_prio = len({info.channel.canonical_name for info in index.values()})
        current_channel = next(iter(index.values())).channel.canonical_name

    for info in index.values():
        # add priority here
        if has_priority:
            if info.channel.canonical_name != current_channel:
                channel_prio -= 1
                current_channel = info.channel.canonical_name
            priority = channel_prio
        else:
            priority = 0
        if has_priority:
            # NOTE: -- this is the whole reason we are vendoring this file --
            # We are patching this from 0 to 1, starting with mamba 0.19
            # Otherwise, test_create::test_force_remove fails :shrug:
            subpriority = 1
        else:
            subpriority = subprio_index
            subprio_index -= 1

        if not context.json:
            log.debug(
                "Channel: %s, platform: %s, prio: %s : %s",
                info.channel,
                info.channel.subdir,
                priority,
                subpriority,
            )
        info.repo.set_priority(priority, subpriority)

    return index


def init_api_context() -> api.Context:
    api_ctx = api.Context()

    # Output params
    # We use this getattr() trick to guarantee backwards compatibility
    # with libmambapy <=1.4.2; 1.4.3+ grouped some settings together
    output_params = getattr(api_ctx, "output_params", api_ctx)
    output_params.json = context.json
    output_params.quiet = context.quiet
    output_params.verbosity = context.verbosity
    api_ctx.set_verbosity(context.verbosity)
    if output_params.json:
        api.cancel_json_output()

    # Prefix params
    prefix_params = getattr(api_ctx, "prefix_params", api_ctx)
    prefix_params.conda_prefix = context.conda_prefix
    prefix_params.root_prefix = context.root_prefix
    prefix_params.target_prefix = context.target_prefix

    # Networking params -- we always operate offline from libmamba's perspective
    remote_fetch_params = getattr(api_ctx, "remote_fetch_params", api_ctx)
    remote_fetch_params.user_agent = context.user_agent
    api_ctx.local_repodata_ttl = context.local_repodata_ttl
    api_ctx.offline = True
    api_ctx.use_index_cache = True

    # General params
    api_ctx.add_pip_as_python_dependency = context.add_pip_as_python_dependency
    api_ctx.always_yes = context.always_yes
    api_ctx.dry_run = context.dry_run
    api_ctx.envs_dirs = context.envs_dirs
    api_ctx.pkgs_dirs = context.pkgs_dirs
    api_ctx.use_lockfiles = False
    api_ctx.use_only_tar_bz2 = context.use_only_tar_bz2

    # Channels and platforms
    api_ctx.platform = context.subdir
    api_ctx.channels = context.channels
    api_ctx.channel_alias = str(_get_base_url(context.channel_alias.url(with_credentials=True)))

    RESERVED_NAMES = {"local", "defaults"}
    additional_custom_channels = {}
    for el in context.custom_channels:
        if context.custom_channels[el].canonical_name not in RESERVED_NAMES:
            additional_custom_channels[el] = _get_base_url(
                context.custom_channels[el].url(with_credentials=True), el
            )
    api_ctx.custom_channels = additional_custom_channels

    additional_custom_multichannels = {}
    for el in context.custom_multichannels:
        if el not in RESERVED_NAMES:
            additional_custom_multichannels[el] = []
            for c in context.custom_multichannels[el]:
                additional_custom_multichannels[el].append(
                    _get_base_url(c.url(with_credentials=True))
                )
    api_ctx.custom_multichannels = additional_custom_multichannels

    api_ctx.default_channels = [
        _get_base_url(x.url(with_credentials=True)) for x in context.default_channels
    ]

    if context.channel_priority is ChannelPriority.STRICT:
        api_ctx.channel_priority = api.ChannelPriority.kStrict
    elif context.channel_priority is ChannelPriority.FLEXIBLE:
        api_ctx.channel_priority = api.ChannelPriority.kFlexible
    elif context.channel_priority is ChannelPriority.DISABLED:
        api_ctx.channel_priority = api.ChannelPriority.kDisabled

    return api_ctx



# Copyright (C) 2019, QuantStack
# Copyright (C) 2022 Anaconda, Inc
# Copyright (C) 2023 conda
# SPDX-License-Identifier: BSD-3-Clause
import argparse
import json
import sys

from conda.base.context import context
from conda.cli import conda_argparse
from conda.common.io import Spinner
from conda.core.prefix_data import PrefixData
from libmambapy import QueryFormat

from .index import LibMambaIndexHelper


def configure_parser(parser: argparse.ArgumentParser):
    package_cmds = argparse.ArgumentParser(add_help=False)
    package_cmds.add_argument("package_query", help="The target package.")
    package_grp = package_cmds.add_argument_group("Subcommand options")
    package_grp.add_argument(
        "-i",
        "--installed",
        action="store_true",
        default=True,
        help=argparse.SUPPRESS,
    )

    package_grp.add_argument(
        "-p",
        "--platform",
        default=context.subdir,
        help="Platform/subdir to search packages for. Defaults to current platform.",
    )
    package_grp.add_argument(
        "--no-installed", action="store_true", help="Do not search currently installed packages."
    )
    package_grp.add_argument(
        "--pretty", action="store_true", help="Prettier output with more details."
    )

    package_grp.add_argument(
        "-a",
        "--all-channels",
        action="store_true",
        help="Look at all channels (for depends / whoneeds).",
    )

    view_cmds = argparse.ArgumentParser(add_help=False)
    view_grp = view_cmds.add_argument_group("Dependency options")
    view_grp.add_argument(
        "-t", "--tree", action="store_true", help="Show dependencies in a tree-like format."
    )
    view_grp.add_argument(
        "--recursive", action="store_true", help="Show dependencies recursively."
    )

    subparser = parser.add_subparsers(dest="subcmd")

    whoneeds = subparser.add_parser(
        "whoneeds",
        help="Show packages that depend on this package.",
        parents=[package_cmds, view_cmds],
    )

    depends = subparser.add_parser(
        "depends",
        help="Show dependencies of this package.",
        parents=[package_cmds, view_cmds],
    )

    search = subparser.add_parser(
        "search",
        help="Show all available package versions.",
        parents=[package_cmds],
    )

    for cmd in (whoneeds, search, depends):
        conda_argparse.add_parser_channels(cmd)
        conda_argparse.add_parser_networking(cmd)
        conda_argparse.add_parser_known(cmd)
        conda_argparse.add_parser_json(cmd)


def repoquery(args):
    if not args.subcmd:
        print("repoquery needs a subcommand (search, depends or whoneeds), e.g.:", file=sys.stderr)
        print("    conda repoquery search python\n", file=sys.stderr)
        return 1

    cli_flags = [getattr(args, attr, False) for attr in ("tree", "recursive", "pretty")]
    if sum([context.json, *cli_flags]) > 1:
        print("Use only one of --json, --tree, --recursive and --pretty.", file=sys.stderr)
        return 1

    if hasattr(args, "channel"):
        channels = args.channel
    else:
        channels = None
    if args.all_channels or (channels is None and args.subcmd == "search"):
        if channels:
            print("WARNING: Using all channels instead of configured channels\n", file=sys.stderr)
        channels = context.channels

    use_installed = args.installed
    if args.no_installed:
        use_installed = False

    # if we're asking for depends and channels are given, disregard
    # installed packages to prevent weird mixing
    if args.subcmd in ("depends", "whoneeds") and use_installed and channels:
        use_installed = False

    if args.subcmd == "search" and not args.installed:
        only_installed = False
    elif args.all_channels or (channels and len(channels)):
        only_installed = False
    else:
        only_installed = True

    if only_installed and args.no_installed:
        print("No channels selected. Use -a to search all channels.", file=sys.stderr)
        return 1

    if use_installed:
        prefix_data = PrefixData(context.target_prefix)
        prefix_data.load()
        installed_records = prefix_data.iter_records()
    else:
        installed_records = ()

    if context.json:
        query_format = QueryFormat.JSON
    elif getattr(args, "tree", None):
        query_format = QueryFormat.TREE
    elif getattr(args, "recursive", None):
        query_format = QueryFormat.RECURSIVETABLE
    elif getattr(args, "pretty", None):
        query_format = QueryFormat.PRETTY
    else:
        query_format = QueryFormat.TABLE

    with Spinner(
        "Collecting package metadata",
        enabled=not context.verbosity and not context.quiet,
        json=context.json,
    ):
        index = LibMambaIndexHelper(
            installed_records=installed_records,
            channels=channels,
            subdirs=(args.platform, "noarch"),
            repodata_fn=context.repodata_fns[-1],
            query_format=query_format,
        )

    result = getattr(index, args.subcmd)(args.package_query, records=False)
    if context.json:
        print(json.dumps(result, indent=2))
    else:
        print(result)
